{ This demonstration program shows real world examples of initial value ordinary
  differential equations (ODE).
  You don't need a degree in civil engineering (or equal) to understand the examples,
  but we will assume the reader (user) has an appropriate amount of pre-knowledge in the
  fields of mathematics and science.


  Author: John Merrit
  Correspondence: John.Merrit@NET4ALL.BE

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 


  References:
  -----------
  The examples used are taken from:
  - Advanced Mathematics for Scientists and Engineers - Murray R. Spiegel
  - Applied Numerical Methods in C - S. Nakamura

  The code implementations are yours truly's.                    }
unit DifEquat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Examples1: TMenuItem;
    Ex11: TMenuItem;
    Temperature1: TMenuItem;
    Example11: TMenuItem;
    Example21: TMenuItem;
    Flow1: TMenuItem;
    Chemistry1: TMenuItem;
    Example31: TMenuItem;
    DampedOscillator1: TMenuItem;
    HarmonicOscillator1: TMenuItem;
    Fallingbody1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure Current1Click(Sender: TObject);
    procedure Temperature1Click(Sender: TObject);
    procedure Example11Click(Sender: TObject);
    procedure Example21Click(Sender: TObject);
    procedure Flow1Click(Sender: TObject);
    procedure Example31Click(Sender: TObject);
    procedure DampedOscillator1Click(Sender: TObject);
    procedure HarmonicOscillator1Click(Sender: TObject);
    procedure Fallingbody1Click(Sender: TObject);
    procedure Chemistry1Click(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses AdvCalc;

{$R *.DFM}

procedure TForm1.Exit1Click(Sender: TObject);
begin
   Close;
end;

{ Example Electric circuit:
                        L
             ---------\/\/---------
            |        I(T) -->      |
            |                      |
                                   /
            E                      / R
                                   /
            |                      |
            |          S           |
             ----------/ ----------

  A circuit with a self-inductance of L = 50H, a resistance of R = 20 Ohm, and a
  voltage source of V = 10V.
  If the switch (S) is closed at T = 0, find the current for 0 < T <= 10 sec.
                                       dI
  The current can be found by solving L-- + R*I(T) = E (1).
                                       dT
  First, we rewrite (1) as   dI     R     E
                             -- = - - I + -
                             dT     L     L

  Data:
       E:= 10; R:= 20; L:= 50; TBegin:= 0; TEnd:= 10;
  In the following, we'll calculate the current at time intervals of 0.1 sec.
  Note that I is replaced by Y (I(0) by Y0).                                    }

procedure TForm1.Current1Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    j, Steps: integer;
    dy: string;
begin
     dy:= '-20*Y/50 + 10/50';
     Steps:= 100;
     StepSize:= 0.1;
     Y0:= 0;
     I:= 0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     I:= InitValueFirstOrderODE(dy, StepSize, I, T);
     Memo1.Lines.Add('Current I: ' + FloatToStr(I) + ' amp');
     end;
end;

{ Example Temperature:
  A metal piece of 0.1kg mass at a temperature of 200C (473K) is suddenly placed
  in a room of temperature 25C, where it is subject to both natural convection cooling
  and radiation heat transfer. Assuming a uniform temperature distribution in the metal,
  calculate the temperature at any given time.

  Temperature equation:
    dY     A
    -- = ----- *[e*s*(297^4 - Y^4) + hc*(297 - Y)]; T(0) = 473, where
    dT   r*c*v

    r = 300 kg/m ( density of metal)
    v = 0.001 m  ( volume of metal )
    A = 0.25 m   ( surface area of metal )
    c = 900 J/kgK ( specific heat of metal )
    hc = 30 J/mK ( heat transfer cofficient )
    e = 0.8       ( emissivity of metal )
    s = 5.67E-8 W/mK^4  ( Stefan-Boltzmann contant )

    The next code will calculate the temperature (Y) at time intervals of 10 sec
    between T(0) = 0 and 180 sec.                                                 }
procedure TForm1.Temperature1Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    j, Steps: integer;
    dy: string;
begin
     dy:= '0.25*(0.8*5.67E-8*(sqr(sqr(297))-sqr(sqr(Y)))+30*(297-Y))/(300*900*0.001)';
     Steps:= 18;
     StepSize:= 10;
     Y0:= 473;
     I:= 0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     I:= InitValueFirstOrderODE(dy, StepSize,Y0, T);
     Y0:= I;
     Memo1.Lines.Add('Temperature: ' + FloatToStr(I) + ' Kelvin');
     end;
end;

{ Radium decays at a rate proportional to the amount present at any time.
  If the half life of radium is T years, determine the amount present after
  t years.
  Equation:
            dY
            -- = - Y*ln(2)/T
            dT
  Initial values (example):
   Y(0) = 500 g
   T = 10000 years
                                                                      }
procedure TForm1.Chemistry1Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    j, Steps: integer;
    dy: string;
begin
     dy:= '-Y*ln(2)/10000';
     Steps:= 20;
     StepSize:= 10;
     Y0:= 500;
     I:= 0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' year(s)');
     I:= InitValueFirstOrderODE(dy, StepSize,Y0, T);
     Y0:= I;
     Memo1.Lines.Add('Amount: ' + FloatToStr(I) + ' g');
     end;
end;

{ Calculate Y(1) by solving Y' = -1/(1 + Y*Y); Y0 = 1 (T0 = 0).
  In other words, we have only one interval.                      }
procedure TForm1.Example11Click(Sender: TObject);
var Y0, T0, StepSize, I: extended;
    dy: string;
begin
     dy:= '-1/(1+sqr(y))';
     StepSize:= 1;
     Y0:= 1;
     T0:= 0;
     I:= InitValueFirstOrderODE(dy, StepSize,Y0, T0);
     Memo1.Lines.Add('Solution y(1) of y'' = -1/(1+sqr(y)); y(0) = 1: ' + FloatToStr(I));
end;

{ Solve Y' = t*Y + 1; Y0 = 0 at t = 1...5.
  Choose the # of intervals as steps, interval size as stepsize.       }
procedure TForm1.Example21Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    j, Steps: integer;
    dy: string;
begin
     dy:= 't*y+1';
     Steps:= 50;
     StepSize:= 0.1;
     Y0:= 0;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     I:= InitValueFirstOrderODE(dy, StepSize, I, T);
     Memo1.Lines.Add('Solution for y'' = t*y + 1; y(0) = 1: ' + FloatToStr(I));
     end;
end;

{ Example Flow:
  A cilindrical tank has 400 litres of a salt solution containing 0.2 kg dissolved
  salt per liter (i.e. 0.2kg salt/l).
  A salt solution of concentration 0.3kg salt/l flows into the tank at 4l/min.
  How much salt is in the tank at any given time if the well-stirred mixture flows
  out at 4 l/min?
  Solution:
  ---------
  The tank contains Y kg salt after T minutes. Then
    Range of change of amount of salt = Rate of entrance - Rate of exit
             dY                kg    4l      Ykg       l
             -- (kg/min) = 0.3 --  * --   -  ---  * 4 ---
             dT                l     min     400l     min
            dY          Y                            kg salt
    Solving -- = 1.2 - --- subject to Y = 400l * 0.2 ------- = 80kg salt at T = 0,
            dT         100                              l
    we find Y = 120 - 40*exp(-T/100).

    The following code calculates Y at times 1...10 minutes.
    You can verify the result(s) using a hand-calculator.                             }
procedure TForm1.Flow1Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    j, Steps: integer;
    dy: string;
begin
     dy:= '1.2 - Y/100';
     Steps:= 10;
     StepSize:= 1;
     Y0:= 80;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' min');
     I:= InitValueFirstOrderODE(dy, StepSize,I, T);
     Memo1.Lines.Add('Amount of salt: ' + FloatToStr(I) + ' kg');
     end;
end;

procedure TForm1.Example31Click(Sender: TObject);
var Y0, T, StepSize, I: extended;
    dy: string;
    j, Steps: integer;
begin
     dy:= 'sqr((t + y))';
     Steps:= 200;
     StepSize:= 0.001;
     Y0:= 1;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     //T:= 0.2;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' min');
     I:= InitValueFirstOrderODE(dy, StepSize,I, T);
     Memo1.Lines.Add('Amount of salt: ' + FloatToStr(I) + ' kg');
     end;
end;

{ Example Damped Oscillating System:
                     /////////////
                           |       |
                           |       | Y
                           \       V
                           /
                           \
                           /
                           |
                           |
                        --- ---
                       |        |
                       |   M    |
                        --- ---
  A mass of 0.5 kg is fixed at the lower end of a massless spring.
  The mass receives resistance R = - B*dY/dT from the air.
  The quation of motion is
    dY     dY
  M --- + B --  + k*Y = 0  (standard equation for a damped oscillator)
    dT      dT

  where Y is the displacement from the static position, k the spring constant equal to
  100 kg/s and B (damping contant) = 10 kg/s.
  }
procedure TForm1.DampedOscillator1Click(Sender: TObject);
var Y0, Z0, T, StepSize, I, V: extended;
    j, Steps: integer;
    dz: string;
begin
     dz:= '-20*z-200*y'; { Note: z = y'; z' = y" }
     Steps:= 30;
     StepSize:= 0.025;
     Y0:= 1;
     Z0:= 0;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     InitValueSecondOrderODE(I, V, dz, StepSize,Y0 , Z0, T);
     Y0:= I;
     Z0:= V;
     Memo1.Lines.Add('Displacement: ' + FloatToStr(I) + ' m');
     end;
end;

{ Same as above but now for a non-damped (B = 0) system.          }
procedure TForm1.HarmonicOscillator1Click(Sender: TObject);
var Y0, Z0, T, StepSize, I, V: extended;
    j, Steps: integer;
    dz: string;
begin
     dz:= '-200*y';
     Steps:= 30;
     StepSize:= 0.025;
     Y0:= 1;
     Z0:= 0;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     InitValueSecondOrderODE(I, V, dz, StepSize,Y0 , Z0, T);
     Y0:= I;
     Z0:= V;
     Memo1.Lines.Add('Displacement: ' + FloatToStr(I) + ' m');
     Memo1.Lines.Add('Speed: ' + FloatToStr(V) + ' m/s');
     end;
end;


{ Example Falling Body:
  An Object is thrown vertically upward from the ground with initial velocity 1960 cm/sec.
  Neglecting air resistance, find the height at any time T.
  By Newton's second law:
       dY
     m ---  = - m*g // m = object's mass ; g ~ 10
       dT

     The initial conditions:
                            Y(0) = 0;
                            V(0) = 1960 cm/sec
  The following code calculates the height at intervals of 0.1 sec.
  When running this program, note the maximum height (after 2 seconds) and the time
  at wich the object touches the ground (after 4 seconds)! }
procedure TForm1.Fallingbody1Click(Sender: TObject);
var Y0, Z0, T, StepSize, I, V: extended;
    j, Steps: integer;
    dz: string;
begin
     dz:= '-980';
     Steps:= 40;
     StepSize:= 0.1;
     Y0:= 0;
     Z0:= 1960;
     I:= Y0;
     for j:= 1 to Steps do
     begin
     T:= j*stepSize;
     Memo1.Lines.Add('Time T: ' + FloatToStr(T) + ' sec');
     InitValueSecondOrderODE(I, V, dz, StepSize,Y0 , Z0, T);
     Y0:= I;
     Z0:= V;
     Memo1.Lines.Add('Height: ' + FloatToStr(I) + ' m');
     Memo1.Lines.Add('Speed: ' + FloatToStr(V) + ' m/s');
     end;
end;




end.
